from distutils.core import setup

setup(
    name='zuice',
    version='0.2.0',
    description='A dependency injection framework for Python',
    author='Michael Williamson',
    author_email='mike@zwobble.org',
    url='https://github.com/mwilliamson/zuice',
    packages=['zuice'],
    keywords="dependency injection di",
)
